<?php
/**
 * License: GPLv3
 * License URI: https://www.gnu.org/licenses/gpl.txt
 * Copyright 2012-2019 Jean-Sebastien Morisset (https://surniaulula.com/)
 */

if ( ! defined( 'ABSPATH' ) ) {
	die( 'These aren\'t the droids you\'re looking for...' );
}

if ( ! class_exists( 'NgfbDeprecate' ) ) {

	class NgfbDeprecate {

		private static $action_query     = 'ngfb-deprecate';
		private static $button_href      = array();
		private static $wpsso_name       = 'WPSSO Core';
		private static $wpsso_ssb_name   = 'WPSSO Social Sharing Buttons';
		private static $wpsso_rrssb_name = 'WPSSO Ridiculously Social Sharing Buttons';
		private static $wpsso_url        = 'https://wpsso.com/extend/plugins/wpsso/free/';
		private static $wpsso_ssb_url    = 'https://wpsso.com/extend/plugins/wpsso-ssb/free/';
		private static $wpsso_rrssb_url  = 'https://wpsso.com/extend/plugins/wpsso-rrssb/free/';

		/**
		 * List plugins in the action name array by their install order.
		 */
		private static $install_slugs = array(
			'wpsso' => array(	// Action name.
				'wpsso' => array(
					'name' => 'WPSSO Core',
					'base' => 'wpsso/wpsso.php',
				),
			),
			'wpsso-ssb' => array(	// Action name.
				'wpsso' => array(
					'name' => 'WPSSO Core',
					'base' => 'wpsso/wpsso.php',
				),
				'wpsso-ssb' => array(
					'name' => 'WPSSO Social Sharing Buttons',
					'base' => 'wpsso-ssb/wpsso-ssb.php',
				),
			),
		);

		/**
		 * List plugins in the action name array by their uninstall order.
		 */
		private static $uninstall_slugs = array(
			'nextgen-facebook' => array(
				'nextgen-facebook-um' => array(
					'name' => 'NextGEN Facebook Update Manager',
					'base' => 'nextgen-facebook-um/nextgen-facebook-um.php',
				),
				'nextgen-facebook' => array(
					'name' => 'NextGEN Facebook',
					'base' => 'nextgen-facebook/nextgen-facebook.php',
				),
			),
		);

		public function __construct() {

			if ( ! is_admin() ) {	// Just in case.
				return;
			}

			add_action( 'ngfb_init_objects', array( __CLASS__, 'setup_variables' ) );

			if ( ! empty( $_GET[ self::$action_query ] ) ) {
				add_action( 'ngfb_init_plugin', array( __CLASS__, 'handle_action_query' ), PHP_INT_MAX );
			} else {
				add_filter( 'current_screen', array( __CLASS__, 'maybe_show_screen_notices' ), PHP_INT_MAX );
			}
		}

		public static function setup_variables() {

			if ( ! class_exists( 'Ngfb' ) ) {	// Just in case.
				return false;
			}

			$ngfb =& Ngfb::get_instance();

			/**
			 * Make sure settings are preserved on uninstall.
			 */
			if ( empty( $ngfb->options[ 'plugin_preserve' ] ) ) {	// Option not enabled.

				$ngfb->options[ 'plugin_preserve' ] = 1;

				$saved = update_option( NGFB_OPTIONS_NAME, $ngfb->options );

				if ( $saved !== true ) {	// Just in case.
					return;
				}
			}

			$ngfb->options[ 'plugin_preserve:is' ] = 'disabled';	// Do not allow changes.

			/**
			 * Create an array of submit button urls.
			 */
			$dashboard_url = get_dashboard_url();

			foreach ( self::$install_slugs as $action_name => $plugin_slugs ) {
				self::$button_href[ $action_name ] = wp_nonce_url( add_query_arg( self::$action_query, $action_name, $dashboard_url ),
					NgfbAdmin::get_nonce_action(), NGFB_NONCE_NAME );
			}
		}

		public static function handle_action_query() {

			if ( empty( $_GET[ self::$action_query ] ) ) {	// Just in case.
				return;
			}

			if ( ! class_exists( 'Ngfb' ) ) {	// Just in case.
				return false;
			}

			$ngfb =& Ngfb::get_instance();

			if ( $ngfb->debug->enabled ) {
				$ngfb->debug->mark();
			}

			$info = $ngfb->cf['plugin']['ngfb'];

			$action_name = SucomUtil::sanitize_key( $_GET[ self::$action_query ] );

			$_SERVER['REQUEST_URI'] = remove_query_arg( array( self::$action_query, NGFB_NONCE_NAME ) );

			/**
			 * Sanitation checks.
			 */
			if ( ! SucomUtil::get_wp_plugin_dir() ) {	// Make sure we have a valid WP_PLUGIN_DIR constant.

				$ngfb->notice->err( sprintf( __( 'The %s constant is not defined or its folder is not writable.', 'nextgen-facebook' ), 'WP_PLUGIN_DIR' ) );

				return;

			} elseif ( empty( $_GET[ NGFB_NONCE_NAME ] ) ) {

				$ngfb->notice->err( __( 'Nonce query value is empty for deprecate action.', 'nextgen-facebook' ) );

				return;

			} elseif ( ! wp_verify_nonce( $_GET[ NGFB_NONCE_NAME ], NgfbAdmin::get_nonce_action() ) ) {

				$ngfb->notice->err( __( 'Nonce token validation failed for deprecate action.', 'nextgen-facebook' ) );

				return;

			} elseif ( empty( self::$install_slugs[ $action_name ] ) ) {

				$ngfb->notice->err( sprintf( __( 'Unknown deprecate action name "%s".', 'nextgen-facebook' ), $action_name ) );

				return;
			}

			/**
			 * Load wp_clean_plugins_cache(), activate_plugin(), deactivate_plugins(), etc.
			 */
			if ( ! function_exists( 'activate_plugin' ) ) {
				require_once trailingslashit( ABSPATH ).'wp-admin/includes/plugin.php';
			}

			/**
			 * Download and install the new plugins.
			 */
			foreach ( self::$install_slugs[ $action_name ] as $plugin_slug => $plugin_info ) {

				/**
				 * Make sure the plugin isn't already active or installed.
				 */
				if ( SucomUtil::plugin_is_active( $plugin_info['base'] ) ) {

					$ngfb->notice->err( sprintf( __( 'Plugin "%s" is already active.', 'nextgen-facebook' ), $plugin_info['name'] ) );

					return;	// Stop on error.

				} elseif ( SucomUtil::plugin_is_installed( $plugin_info['base'] ) ) {

					$ngfb->notice->err( sprintf( __( 'Plugin "%s" is already installed.', 'nextgen-facebook' ), $plugin_info['name'] ) );

					return;	// Stop on error.
				}
				
				/**
				 * Use $unfiltered = true to download and install the version from wordpress.org.
				 * Warning: Does not remove an existing plugin before extracting the zip file.
				 */
				$ret = SucomUtil::download_install_slug( $plugin_slug, true );
				
				if ( is_wp_error( $ret ) ) {

					$ngfb->notice->err( sprintf( __( 'Error installing the "%1$s" plugin: %2$s', 'nextgen-facebook' ), $plugin_info['name'], $ret->get_error_message() ) );
					return;	// Stop on error.
				}
			}

			/**
			 * Must clear the cache to read the new installed plugins.
			 */
			wp_clean_plugins_cache();

			/**
			 * Activate the new plugins.
			 */
			foreach ( self::$install_slugs[ $action_name ] as $plugin_slug => $plugin_info ) {

				/**
				 * $use_cache = false to get the new installed plugins list.
				 */
				if ( ! SucomUtil::plugin_is_installed( $plugin_info['base'], false ) ) {

					$ngfb->notice->err( sprintf( __( 'Plugin "%s" is not installed.', 'nextgen-facebook' ), $plugin_info['name'] ) );

					return;	// Stop on error.
				}

				$ret = activate_plugin( $plugin_info['base'], '', false, true );

				if ( is_wp_error( $ret ) ) {

					$ngfb->notice->err( sprintf( __( 'Error activating the "%1$s" plugin: %2$s', 'nextgen-facebook' ), $plugin_info['name'], $ret->get_error_message() ) );

					return;	// Stop on error.
				}

				$ngfb->notice->upd( sprintf( __( 'The "%s" plugin was successfully installed and activated.', 'nextgen-facebook' ), $plugin_info['name'] ) );
			}

			if ( ! class_exists( 'Wpsso' ) ) {	// Just in case.

				$ngfb->notice->err( __( 'The Wpsso PHP class is required but not loaded.', 'nextgen-facebook' ) );

				return;	// Stop on error.
			}

			/**
			 * Deactivate and uninstall the old plugins.
			 */
			foreach( self::$uninstall_slugs[ 'nextgen-facebook' ] as $plugin_slug => $plugin_info ) {

				/**
				 * Skip plugins that are not installed.
				 */
				if ( ! SucomUtil::plugin_is_installed( $plugin_info['base'] ) ) {
					continue;
				}

				deactivate_plugins( array( $plugin_info['base'] ), true ); // $silent is true

				/**
				 * $use_cache is false to get the new active plugins list.
				 */
				if ( SucomUtil::plugin_is_active( $plugin_info['base'], false ) ) {

					$ngfb->notice->err( sprintf( __( 'Failed to deactivate the "%s" plugin.', 'nextgen-facebook' ), $plugin_info['name'] ) );

					return;	// Stop on error.
				}

				$ret = delete_plugins( array( $plugin_info['base'] ) );

				if ( is_wp_error( $ret ) ) {

					$ngfb->notice->err( sprintf( __( 'Error uninstalling the "%1$s" plugin: %2$s', 'nextgen-facebook' ), $plugin_info['name'], $ret->get_error_message() ) );

					return;	// Stop on error.
				}

				$ngfb->notice->upd( sprintf( __( 'The "%s" plugin was successfully deactivated and uninstalled.', 'nextgen-facebook' ), $plugin_info['name'] ) );
			}

			$user_id = (int) get_current_user_id();

			if ( empty( $user_id ) ) {	// Just in case.
				return;
			}

			$ngfb->util->clear_all_cache();

			wp_redirect( $_SERVER['REQUEST_URI'] );	// Reload current page with wpsso active.
		}

		public static function maybe_show_screen_notices( $screen ) {

			if ( self::can_deprecate_plugin( 'gpl' ) ) {

				self::show_deprecate_gpl_plugin();

			} elseif ( self::can_deprecate_plugin( 'pro' ) ) {

				self::show_deprecate_pro_plugin();
			}

			return $screen;
		}

		private static function can_deprecate_plugin( $pkg_type = 'gpl' ) {

			if ( ! class_exists( 'Ngfb' ) ) {	// Just in case.
				return false;
			}

			$ngfb =& Ngfb::get_instance();

			if ( $ngfb->debug->enabled ) {
				$ngfb->debug->mark();
			}

			$user_id = get_current_user_id();
			$have_p_dir = empty( $ngfb->avail['*']['p_dir'] ) ? false : true;

			if ( $pkg_type === 'gpl' && $have_p_dir ) {	// Pro package is installed.

				if ( $ngfb->debug->enabled ) {
					$ngfb->debug->log( 'exiting early: pro package is installed' );
				}

				return false;

			} elseif ( $pkg_type === 'pro' && ! $have_p_dir ) {	// GPL package is installed.

				if ( $ngfb->debug->enabled ) {
					$ngfb->debug->log( 'exiting early: gpl package is installed' );
				}

				return false;

			} elseif ( empty( $user_id ) ) {

				if ( $ngfb->debug->enabled ) {
					$ngfb->debug->log( 'exiting early: current user id is empty' );
				}

				return false;

			} elseif ( is_multisite() ) {

				if ( $ngfb->debug->enabled ) {
					$ngfb->debug->log( 'exiting early: cannot deprecate a multisite' );
				}

				return false;

			} elseif ( ! is_admin() ) {

				if ( $ngfb->debug->enabled ) {
					$ngfb->debug->log( 'exiting early: must be in admin back-end to deprecate' );
				}

				return false;

			} elseif ( ! current_user_can( 'install_plugins' ) ) {

				if ( $ngfb->debug->enabled ) {
					$ngfb->debug->log( 'exiting early: current user not allowed to install plugins' );
				}

				return false;
			}

			return true;
		}

		private static function show_deprecate_gpl_plugin() {

			if ( ! class_exists( 'Ngfb' ) ) {	// Just in case.
				return false;
			}

			$ngfb =& Ngfb::get_instance();

			if ( empty( self::$button_href ) ) {	// Just in case.
				return;
			}

			$info = $ngfb->cf['plugin']['ngfb'];

			$wpsso_link       = '<a href="' . self::$wpsso_url . '?utm_source=ngfb_gpl_deprecate' . '">' . self::$wpsso_name . '</a>';
			$wpsso_ssb_link   = '<a href="' . self::$wpsso_ssb_url . '?utm_source=ngfb_gpl_deprecate' . '">' . self::$wpsso_ssb_name . '</a>';
			$wpsso_rrssb_link = '<a href="' . self::$wpsso_rrssb_url . '?utm_source=ngfb_gpl_deprecate' . '">' . self::$wpsso_rrssb_name . '</a>';

			$notice_msg = '<h2 class="top">' . sprintf( __( 'The %1$s plugin was deprecated on September 14th 2015.', 'nextgen-facebook' ), $info['name'] ) . '</h2>';

			$notice_msg .= '<p>';

			$notice_msg .= sprintf( __( '%1$s replaced %2$s several years ago - %3$s offers several improvements, new features, and <a href="%4$s">optional Free add-ons</a>.', 'nextgen-facebook' ), $wpsso_link, $info['name'], self::$wpsso_name, 'https://wordpress.org/plugins/search/wpsso/' ) . ' '; 

			$notice_msg .= sprintf( __( 'The social sharing buttons, for example, are distributed as an optional Free add-on.', 'nextgen-facebook' ) ) . ' ';

			$notice_msg .= sprintf( __( 'You can use the traditional %1$s add-on, the latest %2$s add-on, or any other share buttons plugin - or no share buttons at all.', 'nextgen-facebook' ), $wpsso_ssb_link, $wpsso_rrssb_link ) . ' ;-)';

			$notice_msg .= '</p><p>';

			$notice_msg .= sprintf( __( '%1$s also includes support for the new "Gutenberg" block editor in WordPress v5.0 - %2$s does not.', 'nextgen-facebook' ), self::$wpsso_name, $info['name'] ) . ' ';
			
			$notice_msg .= sprintf( __( 'Although the current version of %1$s is stable and working fine, WordPress v5.0 will create some compatibility issues - most notably, the new "Gutenberg" block editor in WordPress v5.0 does not display notices correctly (a different notification system is used), and does not refresh metaboxes after saving a post / page.', 'nextgen-facebook' ), $info['name'] ) . ' ';

			$notice_msg .= '</p><p>';

			$notice_msg .= sprintf( __( 'On April 24th 2018, the WordPress.org plugin review team removed %1$s from the plugin directory and the Free version was officially retired.', 'nextgen-facebook' ), $info['name'] ) . ' ';

			$notice_msg .= '</p><p style="margin-bottom:20px;">';

			$notice_msg .= sprintf( __( 'You should upgrade to the Free version of %2$s immediately - since you\'re moving from the Free version of %1$s to the Free version of %2$s, you will not lose any functionality.', 'nextgen-facebook' ), $info['name'], self::$wpsso_name ) . ' ';

			$notice_msg .= sprintf( __( '%1$s also reads and converts %2$s settings automatically, so no manual changes are required.', 'nextgen-facebook' ), self::$wpsso_name, $info['name'] ) . ' ';

			$notice_msg .= '</p>';

			if ( ! empty( self::$button_href[ 'wpsso-ssb' ] ) ) {

				$notice_msg .= '<p>';

				$notice_msg .= '<input style="width:100%;" type="button" class="button-primary" value="' .  sprintf( __( 'Replace %1$s by %2$s and %3$s', 'nextgen-facebook' ), $info['name'], self::$wpsso_name, self::$wpsso_ssb_name ) . '" onclick="location.href=\'' . esc_url_raw( self::$button_href['wpsso-ssb'] ) . '\';" />';

				$notice_msg .= '</p>';
			}

			if ( ! empty( self::$button_href[ 'wpsso' ] ) ) {

				$notice_msg .= '<p>';

				$notice_msg .= '<input style="width:100%;" type="button" class="button-secondary" value="' . sprintf( __( 'Replace %1$s by %2$s without the social sharing buttons', 'nextgen-facebook' ), $info['name'], self::$wpsso_name ) . '" onclick="location.href=\'' . esc_url_raw( self::$button_href['wpsso'] ) . '\';" />';

				$notice_msg .= '</p>';
			}

			if ( ! empty( $notice_msg ) ) {

				$user_id      = get_current_user_id();
				$notice_key   = 'ngfb-gpl-deprecation-notice';
				$dismiss_time = false;

				$ngfb->notice->inf( $notice_msg, $user_id, $notice_key, $dismiss_time );
			}
		}

		private static function show_deprecate_pro_plugin() {

			if ( ! class_exists( 'Ngfb' ) ) {	// Just in case.
				return false;
			}

			$ngfb =& Ngfb::get_instance();

			if ( empty( self::$button_href ) ) {	// Just in case.
				return;
			}

			$info = $ngfb->cf['plugin']['ngfb'];

			$wpsso_link       = '<a href="' . self::$wpsso_url . '?utm_source=ngfb_pro_deprecate' . '">' . self::$wpsso_name . '</a>';
			$wpsso_ssb_link   = '<a href="' . self::$wpsso_ssb_url . '?utm_source=ngfb_pro_deprecate' . '">' . self::$wpsso_ssb_name . '</a>';
			$wpsso_rrssb_link = '<a href="' . self::$wpsso_rrssb_url . '?utm_source=ngfb_pro_deprecate' . '">' . self::$wpsso_rrssb_name . '</a>';

			$notice_msg = '<h2 class="top">' . sprintf( __( 'The %1$s plugin was deprecated on September 14th 2015.', 'nextgen-facebook' ), $info['name'] ) . '</h2>';

			$notice_msg .= '<p>';

			$notice_msg .= sprintf( __( 'On September 14th 2015, a notice was added to the plugin description to inform users that %1$s was replaced by the %2$s plugin.', 'nextgen-facebook' ), $info['name'], $wpsso_link ) . ' ';

			$notice_msg .= sprintf( __( 'Using %1$s (instead of %2$s) was recommended as a better alternative for new installations and Pro version purchases.', 'nextgen-facebook' ), self::$wpsso_name, $info['name'] ) . ' ';

			$notice_msg .= '</p><p>';

			$notice_msg .= sprintf( __( 'On April 24th 2018, the WordPress.org plugin review team removed %1$s from the plugin directory and the Free version was officially retired.', 'nextgen-facebook' ), $info['name'] ) . ' ';

			$notice_msg .= '</p><p>';

			$notice_msg .= sprintf( __( 'On October 24th 2018, the Pro version of %1$s was retired. %1$s has reached it\'s end-of-life.', 'nextgen-facebook' ), $info['name'] ) . ' ';

			$notice_msg .= '</p><p>';

			$notice_msg .= sprintf( __( 'The Pro version of %1$s is no longer being developed and should be uninstalled before upgrading to WordPress v5.0.', 'nextgen-facebook' ), $info['name'] ) . ' ';

			$notice_msg .= sprintf( __( 'Although the current version of %1$s is stable and working fine, WordPress v5.0 will create some compatibility issues - most notably, the new "Gutenberg" block editor in WordPress v5.0 does not display notices correctly (a different notification system is used), and does not refresh metaboxes after saving a post / page.', 'nextgen-facebook' ), $info['name'] ) . ' ';


			$notice_msg .= '</p><p style="margin-bottom:20px;">';

			$notice_msg .= sprintf( __( '%1$s reads and converts %2$s settings automatically, so if you desire to install the Free version of %1$s, no manual changes are required.', 'nextgen-facebook' ), self::$wpsso_name, $info['name'] ) . ' ';

			$notice_msg .= sprintf( __( 'Note that Pro version licenses purchased for %1$s are for %1$s Pro only - authentication IDs for %1$s Pro are not compatible with any other Pro / Premium plugins - using the Free version of %2$s may mean losing some %1$s Pro features that you may have been using.', 'nextgen-facebook' ), $info['name'], self::$wpsso_name ) . ' ';

			$notice_msg .= sprintf( __( 'If you wish to keep using the %1$s Pro plugin, you may dismiss this notice for %2$s.', 'nextgen-facebook' ), $info['name'], __( 'three months', 'nextgen-facebook' ) ) . ' ';

			$notice_msg .= '</p>';

			if ( ! empty( self::$button_href[ 'wpsso-ssb' ] ) ) {

				$notice_msg .= '<p>';

				$notice_msg .= '<input style="width:100%;" type="button" class="button-primary" value="' .  sprintf( __( 'Replace %1$s Pro by the Free version of %2$s and %3$s', 'nextgen-facebook' ), $info['name'], self::$wpsso_name, self::$wpsso_ssb_name ) . '" onclick="location.href=\'' . esc_url_raw( self::$button_href['wpsso-ssb'] ) . '\';" />';

				$notice_msg .= '</p>';
			}

			if ( ! empty( $notice_msg ) ) {

				$user_id      = get_current_user_id();
				$notice_key   = 'ngfb-pro-deprecation-notice';
				$dismiss_time = 3 * MONTH_IN_SECONDS;

				$ngfb->notice->inf( $notice_msg, $user_id, $notice_key, $dismiss_time );
			}
		}
	}

	new NgfbDeprecate();	// Instantiate.
}
